<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Requests\API\File\DownloadZipRequest;
use App\Http\Requests\API\File\FileUploadRequest;
use App\Http\Requests\API\File\KeepDirectoryUploadRequest;
use App\Http\Resources\API\FileByObjectResource;
use App\Http\Resources\API\FileDownloadResource;
use App\Http\Resources\API\FileUploadSuccessResource;
use App\Models\Enums\FileObjectType;
use App\Models\File;
use App\Models\Folder;
use App\Services\File\BIM\BIMFactory;
use App\Services\File\DownloadService;
use App\Services\File\FileHideService;
use App\Services\File\Upload\FilesUploadService;
use App\Services\File\Upload\KeepDirectoryUploadService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Storage;

class FileController extends Controller
{

    public function download(string $id)
    {
        $file = File::query()->findOrFail($id);

        $fileObjectType = FileObjectType::from($file->object_type);
        $object = $fileObjectType->modelBuilderAllowed($file->object_id)->find($file->object_id);
        if(! $object){
            return $this->badRequest(sprintf("File ID: %s, no permission to access", $file->id));
        }

        return new FileDownloadResource($file);
    }

    public function changeName(Request $request,string $id)
    {
        $isAction = $request->input('is_action', null);
        $result = ($isAction === '0') ? true : ($isAction === '1' ? false : false);

        $file = File::query()->findOrFail($id);

        $fileObjectType = FileObjectType::from($file->object_type);
        $object = $fileObjectType->modelBuilderAllowed($file->object_id,$result)->find($file->object_id);
        if(! $object){
            return $this->badRequest(sprintf("File ID: %s, no permission to access", $file->id));
        }

        File::query()->where('title',$file->title)
            ->where('object_type',$file->object_type)
            ->where("object_id", $file->object_id)
            ->update(['title' => $request->get('title')]);

        return $this->noContent();
    }

    public function destroy(Request $request,string $id){
        $isAction = $request->input('is_action', null);
        $result = ($isAction === '0') ? true : ($isAction === '1' ? false : false);

        $file = File::query()->findOrFail($id);

        $fileObjectType = FileObjectType::from($file->object_type);
        $object = $fileObjectType->modelBuilderAllowed($file->object_id,$result)->find($file->object_id);
        if(! $object){
            return $this->badRequest(sprintf("File ID: %s, no permission to access", $file->id));
        }

        $files = File::query()
            ->where('title',$file->title)
            ->where('object_type',$file->object_type)
            ->where("object_id", $file->object_id)
            ->get();

        File::query()->whereIn("id", $files->pluck("id")->toArray())->delete();

        Storage::delete($files->pluck("pathname")->toArray());

        $filesSize = $files->sum("size");

        $company = Auth::user()->company;
        $company->decrement("used_storage_capacity", $filesSize);
        $company->save();

        return $this->noContent();
    }

    public function downloadZip(DownloadZipRequest $request, DownloadService $service)
    {
        return $this->success([
            'data' => $service->downloadByIds($request->get("ids")),
        ]);
    }

    public function downloadAllLatest(DownloadService $service, string $objectType, string $objectId)
    {
        return $this->success([
            'data' => $service->downloadAllLatest($objectType, $objectId),
        ]);
    }

    public function downloadShareFile(DownloadService $service, string $uuid)
    {
        return $this->success([
            'data' => $service->downloadShareFile($uuid),
        ]);
    }

    /**
     * 文件上传
     *
     * @param FileUploadRequest $request
     * @return FileUploadSuccessResource|\Illuminate\Http\JsonResponse
     */
    public function upload(FileUploadRequest $request)
    {
        set_time_limit(0);

        $uploadedFiles = (new FilesUploadService($request))->upload();

        return $this->success([
            'data' => $uploadedFiles
        ]);

    }


    /**
     * 文件夹上传
     *
     * @param KeepDirectoryUploadRequest $request
     * @return FileUploadSuccessResource|\Illuminate\Http\JsonResponse
     */
    public function keepDirectoryUpload(KeepDirectoryUploadRequest $request)
    {
        set_time_limit(0);

        $uploadedFiles = (new KeepDirectoryUploadService($request))->upload();

        return $this->success([
            'data' => $uploadedFiles
        ]);
    }

    public function byObject(string $objectType, string $objectId)
    {
        $fileObjectType = FileObjectType::from($objectType);
        $fileObjectType->modelBuilderAllowed($objectId)->findOrFail($objectId);

        $files = File::query()
            ->with(['createdBy'])
            ->where('object_type', $objectType)
            ->where('object_id', $objectId)
            ->orderByDesc("version")
            ->where("source", 1)
            ->get();


        $items = [];
        foreach ($files->groupBy("title") as $fileItems) {
            $item = (new FileByObjectResource($fileItems->first()))->toArray(request());
            $item['children'] = [];

            foreach ($fileItems as $fileItem) {
                $item['children'][] = new FileByObjectResource($fileItem);
            }

            $items[] = $item;
        }

        return $this->success([
            'data' => $items
        ]);
    }

    /**
     * 文件隐藏
     *
     * @param string $id
     * @param FileHideService $service
     * @return \Illuminate\Http\Response
     */
    public function hide(string $id, FileHideService $service){
        $service->hide($id);

        return $this->noContent();
    }

    public function bimView(string $id)
    {
        $file = File::query()->where('is_bim', 1)->findOrFail($id);

        $fileObjectType = FileObjectType::from($file->object_type);
        $object = $fileObjectType->modelBuilderAllowed($file->object_id)->find($file->object_id);
        if(! $object){
            return $this->badRequest(sprintf("File ID: %s, no permission to access", $file->id));
        }

        $bimFile = $file->bimFile;
        if($bimFile->convert_status===5){
            $result = BIMFactory::make($bimFile->bim_driver)->viewDataSetModel([
                $bimFile->bim_data_set_id
            ]);

            return $this->success([
                'data' => [
                    'bim_driver' => $bimFile->bim_driver,
                    'bim_view' => $result
                ]
            ]);
        }else{
            return $this->badRequest("BIM is in the process of conversion. Please wait a moment");
        }

    }
}
